-----------------------------------------------------------------------------------------------
-- Client Lua Script for DeathImproved
-- Copyright (c) NCsoft. All rights reserved
-- Modified by Nahbyr -- http://www.curse.com/private-messages/send?recipient=Nahbyr
-----------------------------------------------------------------------------------------------

-----------------------------------------------------------------------------------------------
-- TODO: See if you can disable OnUnitCreated and OnUnitDestroyed if incoming resurrect was disabled.
-----------------------------------------------------------------------------------------------

require "Window"
require "Tooltip"
require "XmlDoc"
require "GameLib"
require "MatchingGame"
require "Unit"			-- Modified

---------------------------------------------------------------------------------------------------
-- DeathImproved module definition
---------------------------------------------------------------------------------------------------
local DeathImproved  = {}

---------------------------------------------------------------------------------------------------
-- local constants
---------------------------------------------------------------------------------------------------
local RezType =
{
	Here = 1,
	Eldan = 2,
	SpellCasterLocation = 4,
	ExitInstance = 32
}

local kcrCanResurrectButtonColor = CColor.new(1, 1, 1, 1)
local kcrCannotResurrectButtonColor = CColor.new(.6, .6, .6, 1)
local kcrCanResurrectTextColor = ApolloColor.new("ff9aaea3")
local kcrCannotResurrectTextColor = CColor.new(.3, .3, .3, 1)

local knSaveVersion = 1


---------------------------------------------------------------------------------------------------
-- Testing VARs
---------------------------------------------------------------------------------------------------



---------------------------------------------------------------------------------------------------
-- DeathImproved functions
---------------------------------------------------------------------------------------------------
function DeathImproved:new(o)
	o = o or {}
	setmetatable(o, self)
	self.__index = self

	o.wndResurrect = nil
	o.nGoldPenalty = 0
	o.bEnableRezHere = true
	o.bEnableCasterRez = false
	o.bHasCasterRezRequest = false
	o.nRezCost = 0
	o.fTimeBeforeRezable = 0
	o.fTimeBeforeWakeHere = 0

	o.arGroupMembers = {}					-- Modified
	o.bResurrectIncoming = false			-- Modified
	o.bIncRes = true						-- Modified
	o.bGroupOnly = false					-- Modified
	
	return o
end

function DeathImproved:Init()
	Apollo.RegisterAddon(self)
end

---------------------------------------------------------------------------------------------------
-- EventHandlers
---------------------------------------------------------------------------------------------------
function DeathImproved:OnLoad()
	self.xmlDoc = XmlDoc.CreateFromFile("DeathImproved.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
	
end


function DeathImproved:OnDocLoaded()
--	self.Rover = Apollo.GetAddon("Rover")			-- Modified -- For debug only
	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
		Apollo.RegisterSlashCommand("death",      "OnSlashCmd", self)			-- Modified
		Apollo.RegisterTimerHandler("TenthSecTimer", "OnTenthSecTimer", self)
		
		Apollo.RegisterEventHandler("ShowResurrectDialog", 		"OnShowResurrectDialog", self)
		Apollo.RegisterEventHandler("ShowIncapacitationBar", 	"ShowIncapacitationBar", self)
		Apollo.RegisterEventHandler("HideIncapacitationBar", 	"HideIncapacitationBar", self)
		Apollo.RegisterEventHandler("CasterResurrectedPlayer", 	"CasterResurrectedPlayer", self)
		Apollo.RegisterEventHandler("ForceResurrect", 			"OnForcedResurrection", self)
		Apollo.RegisterEventHandler("ScriptResurrect", 			"OnScriptResurrection", self)
		Apollo.RegisterEventHandler("MatchExited", 				"OnForcedResurrection", self)
		Apollo.RegisterEventHandler("PVPDeathmatchPoolUpdated", "OnPVPDeathmatchPoolUpdated", self)
		Apollo.RegisterEventHandler("CharacterCreated", 		"OnCharacterCreated", self)
		
		Apollo.RegisterEventHandler("UnitCreated", 					"OnUnitCreated", self)			-- Modified
		Apollo.RegisterEventHandler("UnitDestroyed",		"OnUnitDestroyed", self)			-- Modified
		Apollo.RegisterEventHandler("VarChange_FrameCount", 		"OnFrame", self)			-- Modified
				
		Apollo.RegisterEventHandler("WindowManagementReady", 		"OnWindowManagementReady", self)			-- Modified

		self.wndResurrect = Apollo.LoadForm(self.xmlDoc, "ResurrectDialog", nil, self)
		self.wndResurrect:Show(false)

		self.wndExitConfirm = Apollo.LoadForm(self.xmlDoc, "ExitInstanceDialog", nil, self)
		self.wndExitConfirm:Show(false)
		
		self.wndResurrectHereConfirm = Apollo.LoadForm(self.xmlDoc, "ResurrectHereDialog", nil, self)			-- Modified
		self.wndResurrectHereConfirm:Show(false)			-- Modified
		
		self.wndDeathOptions = Apollo.LoadForm(self.xmlDoc, "DeathOptionsForm", nil, self)			-- Modified
		self.wndDeathOptions:Show(false)			-- Modified
		
		if self.locSavedWindowLoc then			-- Modified
			self.wndResurrect:MoveToLocation(self.locSavedWindowLoc)			-- Modified
		end			-- Modified
		
		self.xmlDoc = nil
		
		self.nTimerProgress = nil
		self.bDead = false
		
		Apollo.CreateTimer("TenthSecTimer", 0.10, true)
		Apollo.StopTimer("TenthSecTimer")

		self.monPlayerCash = self.wndResurrect:FindChild("ResurrectDialog.PlayerCash")
		if GameLib.GetPlayerUnit() then
			self:OnCharacterCreated()
		end
		
	end
end

------------------------------------------------------------------------------------
---------------------- Modified - Options window -----------------------------------
------------------------------------------------------------------------------------
function DeathImproved:OnWindowManagementReady()
	-- Move to saved location
	if self.locSavedOptionsLoc then
		self.wndDeathOptions:MoveToLocation(self.locSavedOptionsLoc)
	end

	local wndOptionsContainer  = self.wndDeathOptions:FindChild("DeathOptionsContainer")

	self.wndDeathOptions:Show(false)

	-- Confirmation Window Option
	if self.bConfirmWindowFilter == nil then
		self.bConfirmWindowFilter = true
	end
	
	wndOptionsContainer:FindChild("ConfirmationOn"):SetData(true)
	wndOptionsContainer:FindChild("ConfirmationOff"):SetData(false)
	wndOptionsContainer:FindChild("ConfirmationOn"):SetCheck(self.bConfirmWindowFilter) -- Default
	wndOptionsContainer:FindChild("ConfirmationOff"):SetCheck(not self.bConfirmWindowFilter)
	
	-- Show incoming resurrect timer Option
	if self.bIncRes == nil then
		self.bIncRes = true
	end
	
	wndOptionsContainer:FindChild("IncResOn"):SetData(true)
	wndOptionsContainer:FindChild("IncResOff"):SetData(false)
	wndOptionsContainer:FindChild("IncResOn"):SetCheck(self.bIncRes) -- Default
	wndOptionsContainer:FindChild("IncResOff"):SetCheck(not self.bIncRes)
	
	-- Show incoming resurrect timer for group members only Option
	if self.bGroupOnly == nil then
		self.bGroupOnly = true
	end
	
	wndOptionsContainer:FindChild("GroupOn"):SetData(true)
	wndOptionsContainer:FindChild("GroupOff"):SetData(false)
	wndOptionsContainer:FindChild("GroupOn"):SetCheck(not self.bGroupOnly)
	wndOptionsContainer:FindChild("GroupOff"):SetCheck(self.bGroupOnly) -- Default
	
	--------------------- OnRestore section ----------------------
	if self.wndDeathOptions then
		self.wndDeathOptions:FindChild("ConfirmationOn"):SetCheck(self.bConfirmWindowFilter)
		self.wndDeathOptions:FindChild("ConfirmationOff"):SetCheck(not self.bConfirmWindowFilter)
		self.wndDeathOptions:FindChild("IncResOn"):SetCheck(self.bIncRes)
		self.wndDeathOptions:FindChild("IncResOff"):SetCheck(not self.bIncRes)
		self.wndDeathOptions:FindChild("GroupOn"):SetCheck(not self.bGroupOnly)
		self.wndDeathOptions:FindChild("GroupOff"):SetCheck(self.bGroupOnly)
	end
end	

function DeathImproved:OnSave(eType)
	if eType ~= GameLib.CodeEnumAddonSaveLevel.Account then
		return
	end
	
	local tSaveData = 
	{
		bCasterRezzed = self.bHasCasterRezRequest,
		tWindowLocation = self.wndResurrect and self.wndResurrect:GetLocation():ToTable() or self.locSavedWindowLoc:ToTable(),
		tOptionsLocation = self.wndDeathOptions and self.wndDeathOptions:GetLocation():ToTable() or self.locSavedOptionsLoc:ToTable(),
		nSaveVersion = knSaveVersion,
		bConfirmWindowFilter  = self.bConfirmWindowFilter,			-- Modified
		bIncRes = self.bIncRes,			-- Modified
		bGroupOnly = self.bGroupOnly		-- Modified
	}
	return tSaveData
end

function DeathImproved:OnRestore(eType, tSavedData)
	self.tSavedData = tSavedData
	if tSavedData and tSavedData.nSaveVersion == knSaveVersion then
		self.bHasCasterRezRequest = tSavedData.bCasterRezzed
		self.locSavedWindowLoc = WindowLocation.new(tSavedData.tWindowLocation)
		self.locSavedOptionsLoc = WindowLocation.new(tSavedData.tOptionsLocation)
		if tSavedData.bConfirmWindowFilter ~= nil then			-- Modified
			self.bConfirmWindowFilter = tSavedData.bConfirmWindowFilter			-- Modified
		end			-- Modified
		if tSavedData.bIncRes ~= nil then			-- Modified
			self.bIncRes = tSavedData.bIncRes			-- Modified
		end			-- Modified
		if tSavedData.bGroupOnly ~= nil then			-- Modified
			self.bGroupOnly = tSavedData.bGroupOnly			-- Modified
		end			-- Modified
	end
end

---------------------------------------------------------------------------------------------------
-- Interface
---------------------------------------------------------------------------------------------------
function DeathImproved:OnCharacterCreated()
	local unitPlayer = GameLib.GetPlayerUnit()
	
	if unitPlayer:IsDead() == false then
		return
	end
	
	self.bDead = true
	self.fTimeBeforeRezable = GetDeathPenalty() 
	self.fTimeBeforeWakeHere = GetWakeHereTime() 
	self.fTimeBeforeForceRez = GetForceRezTime() 
	self.nRezCost = GetRezCost() 
	self.bEnableRezHere = GetRezOptionWakeHere() 
	self.bEnableRezHoloCrypt = GetRezOptionHolocrypt() 
	self.bEnableRezExitInstance = GetRezOptionExitInstance()
	self.bEnableCasterRez = GetRezOptionAcceptCasterRez() 
	
	self:OnShowResurrectDialog(self.bDead, self.bEnableRezHere, self.bEnableRezHoloCrypt, self.bEnableRezExitInstance, self.bEnableCasterRez, self.bHasCasterRezRequest, self.nRezCost, self.fTimeBeforeRezable, self.fTimeBeforeWakeHere, self.fTimeBeforeForceRez)
end

function DeathImproved:OnShowResurrectDialog(bPlayerIsDead, bEnableRezHere, bEnableRezHoloCrypt, bEnableRezExitInstance, bEnableCasterRez, bHasCasterRezRequest, nRezCost, fTimeBeforeRezable, fTimeBeforeWakeHere, fTimeBeforeForceRez)
	Apollo.StartTimer("TenthSecTimer")

	self.bEnableRezHere = bEnableRezHere
	self.bEnableRezHoloCrypt = bEnableRezHoloCrypt
	self.bEnableRezExitInstance = bEnableRezExitInstance
	self.bEnableCasterRez = bEnableCasterRez or false
	self.bHasCasterRezRequest = bHasCasterRezRequest
	self.bDead = bPlayerIsDead
	self.nRezCost = nRezCost
	self.fTimeBeforeRezable = fTimeBeforeRezable
	self.fTimeBeforeWakeHere = fTimeBeforeWakeHere
	self.fTimeBeforeForceRez = fTimeBeforeForceRez

	if self.bDead == false then
		self.wndExitConfirm:Show(false)	
		self.wndResurrect:Show(false)
		self.wndResurrectHereConfirm:Show(false)			-- Modified
		self.nTimerProgress = nil
		return
	end
	
	--hide and format everything
	self.wndExitConfirm:Show(false)	
	self.wndResurrectHereConfirm:Show(false)
	self.wndResurrect:FindChild("ResurrectDialog.Caster"):Show(false)
	self.wndResurrect:FindChild("ResurrectDialog.Here"):Show(false)
	self.wndResurrect:FindChild("ResurrectDialog.Eldan"):Show(false)
	self.wndResurrect:FindChild("ResurrectDialog.ExitInstance"):Show(false)
	self.wndResurrect:FindChild("ResurrectDialog.EldanSmall"):Show(false)
	self.wndResurrect:FindChild("ResurrectDialog.ExitInstanceSmall"):Show(false)	
	self.wndResurrect:FindChild("ArtTimeToRezHere"):SetText(Apollo.GetString("CRB_Time_Remaining_2"))
	self.wndResurrect:FindChild("ArtTimeToRezHere"):Show(true)
	self.wndResurrect:FindChild("ForceRezText"):Show(false)
	
	self.wndResurrect:Show(true)
end


------------------------------------//------------------------------
function DeathImproved:OnTenthSecTimer() -- there's no reason we can't run the whole thing off of this
	if self.bDead ~= true or not self.wndResurrect:IsVisible() then
		self.bHasCasterRezRequest = false
		return
	end
	
	-- update all of our timers
	self.fTimeBeforeWakeHere = self.fTimeBeforeWakeHere - 100
	self.fTimeBeforeForceRez = self.fTimeBeforeForceRez - 100
	self.fTimeBeforeRezable = self.fTimeBeforeRezable - 100
			
	if self.fTimeBeforeRezable > 0 then -- this timer takes precendence over everything. if it has a count, the player can't do anything
		local strTimeBeforeRezableFormatted = self:HelperCalcTimeSecondsMS(self.fTimeBeforeRezable)
		self.wndResurrect:FindChild("ResurrectDialog.Timer"):SetText(strTimeBeforeRezableFormatted .. Apollo.GetString("CRB__seconds"))
		self.wndResurrect:FindChild("ResurrectDialog.Timer"):Show(true)
	else
		local tMatchInfo = MatchingGame.GetPVPMatchState()
		if tMatchInfo ~= nil then
			if tMatchInfo.eRules == MatchingGame.Rules.DeathmatchPool then
				if (tMatchInfo.eMyTeam == MatchingGame.Team.Team1 and tMatchInfo.tLivesRemaining.nTeam1 == 0) or (tMatchInfo.eMyTeam == MatchingGame.Team.Team2 and tMatchInfo.tLivesRemaining.nTeam2 == 0) then
					self.wndResurrect:FindChild("ResurrectDialog.Timer"):SetText(Apollo.GetString("Death_NoLives"))
					self.wndResurrect:FindChild("ResurrectDialog.Timer"):Show(true)	
					self.wndResurrect:FindChild("ForceRezText"):Show(false)
					self.wndResurrect:FindChild("ResurrectDialog.Here"):Show(false)
					self.wndResurrect:FindChild("ResurrectDialog.Eldan"):Show(false)
					self.wndResurrect:FindChild("ResurrectDialog.EldanSmall"):Show(false)
					self.wndResurrect:FindChild("ResurrectDialog.ExitInstanceSmall"):Show(false)		
					self.wndResurrect:FindChild("ResurrectDialog.Caster"):Show(false)	
					return
				end
			elseif tMatchInfo.eRules == MatchingGame.Rules.WaveRespawn then
				self.wndResurrect:Show(false)
				return
			end
		end
	
		self.wndResurrect:FindChild("ArtTimeToRezHere"):Show(false)
		self.wndResurrect:FindChild("ResurrectDialog.Timer"):Show(false)
		
		self.wndResurrect:FindChild("ForceRezText"):Show(self.bEnableRezHoloCrypt)
		self.wndResurrect:FindChild("ResurrectDialog.Here"):Show(self.bEnableRezHere)
		self.wndResurrect:FindChild("ResurrectDialog.Eldan"):Show(self.bEnableRezHoloCrypt and not self.bEnableRezExitInstance)
		self.wndResurrect:FindChild("ResurrectDialog.ExitInstance"):Show(not self.bEnableRezHoloCrypt and self.bEnableRezExitInstance)
		self.wndResurrect:FindChild("ResurrectDialog.EldanSmall"):Show(self.bEnableRezHoloCrypt and self.bEnableRezExitInstance)
		self.wndResurrect:FindChild("ResurrectDialog.ExitInstanceSmall"):Show(self.bEnableRezHoloCrypt and self.bEnableRezExitInstance)		
		self.wndResurrect:FindChild("ResurrectDialog.Caster"):Show(self.bEnableCasterRez and self.bHasCasterRezRequest)	
		
		local nLeft, nTop, nRight, nBottom = self.wndResurrect:GetAnchorOffsets()
		-- Modified - Show Resurrect at caster if option was selected and hide Wake Here if Resurrect at caster is shown
		if self.wndResurrect:FindChild("ResurrectDialog.Caster"):IsShown() then
			self.wndResurrect:FindChild("ResurrectIncoming.Timer"):Show(false)
			if self.wndResurrect:FindChild("ResurrectDialog.Here"):IsShown() then
				self.wndResurrect:FindChild("ResurrectDialog.Here"):Show(false)
				self.wndResurrect:FindChild("ResurrectDialog.Caster"):SetAnchorOffsets(70, 165, -70, 238)
			elseif self.wndResurrect:FindChild("ResurrectDialog.EldanSmall"):IsShown() then
				self.wndResurrect:FindChild("ResurrectDialog.EldanSmall"):Show(false)
				self.wndResurrect:FindChild("ResurrectDialog.Caster"):SetAnchorOffsets(71, 103, -69, 176)
			end
		-- Modified - Hide Wake Here button if option to show Caster resurrect timer was selected and Caster is resurrecting
		elseif self.bResurrectIncoming then
			self.wndResurrect:FindChild("ResurrectDialog.Here"):Show(false)
			self.wndResurrect:FindChild("ResurrectDialog.EldanSmall"):Show(false)
		else
			self.wndResurrect:SetAnchorOffsets(nLeft, nTop, nRight, nTop + 308)
		end
		
		-- set up rez here
		if self.wndResurrect:FindChild("ResurrectDialog.Here"):IsShown() then
			local wndBtn = self.wndResurrect:FindChild("ResurrectDialog.Here")
			wndBtn:FindChild("ResurrectDialog.Cash"):SetAmount(self.nRezCost)
						
			if self.fTimeBeforeWakeHere <= 0 then  -- ready to go
				local bCanAfford = self.nRezCost <= GameLib.GetPlayerCurrency():GetAmount()
				wndBtn:Enable(bCanAfford)
				wndBtn:FindChild("WakeHereText"):SetText(Apollo.GetString("Death_WakeHere"))
				wndBtn:FindChild("WakeHereCooldownText"):SetText("")
				wndBtn:FindChild("ResurrectDialog.Cash"):Show(true)
				wndBtn:FindChild("CostLabel"):Show(true)
				if bCanAfford then
					self.wndResurrect:FindChild("ResurrectDialog.Here"):SetBGColor(kcrCanResurrectButtonColor)
					wndBtn:FindChild("WakeHereText"):SetTextColor(kcrCanResurrectTextColor)
					wndBtn:FindChild("ResurrectDialog.Cash"):SetTextColor(ApolloColor.new("white"))
				else -- not enough money
					self.wndResurrect:FindChild("ResurrectDialog.Here"):SetBGColor(kcrCannotResurrectButtonColor)
					wndBtn:FindChild("WakeHereText"):SetTextColor(kcrCannotResurrectTextColor)	
					wndBtn:FindChild("ResurrectDialog.Cash"):SetTextColor(ApolloColor.new("red"))						
				end	
			else -- still cooling down
				wndBtn:Enable(false)
				local strCooldownFormatted = self:HelperCalcTimeMS(self.fTimeBeforeWakeHere)
				wndBtn:FindChild("WakeHereCooldownText"):SetText(String_GetWeaselString(Apollo.GetString("Death_CooldownTimer"), strCooldownFormatted))
				wndBtn:FindChild("ResurrectDialog.Cash"):Show(false)
				wndBtn:FindChild("CostLabel"):Show(false)
				self.wndResurrect:FindChild("ResurrectDialog.Here"):SetBGColor(kcrCannotResurrectButtonColor)
				wndBtn:FindChild("WakeHereText"):SetTextColor(kcrCannotResurrectTextColor)					
			end
		end	
	end
		
	if self.fTimeBeforeForceRez > 0 then
		local strTimeFormatted = self:HelperCalcTimeMS(self.fTimeBeforeForceRez)
		self.wndResurrect:FindChild("ForceRezText"):SetText(String_GetWeaselString(Apollo.GetString("Death_AutoRelease"), strTimeFormatted))
	end

	self.monPlayerCash:SetAmount(GameLib.GetPlayerCurrency():GetAmount(), false)
	
end

------------------------------------//------------------------------

function DeathImproved:CasterResurrectedPlayer(strCasterName)
	self.bHasCasterRezRequest = true;
end

------------------------------------------------------------------------------------------
-------------------- Modified - Show Confirmation window if option was selected ----------
------------------------------------------------------------------------------------------
function DeathImproved:OnResurrectHere()
	self.wndResurrectHereConfirm:Show(self.bConfirmWindowFilter)			-- Modified
	
	-- Modified - If option wasn't selected do like the default Add-on
	if not self.bConfirmWindowFilter then
		if GameLib.GetPlayerUnit() ~= nil then
			GameLib.GetPlayerUnit():Resurrect(RezType.Here, 0)
		end
		
		self.bDead = false
	
		Apollo.StopTimer("TenthSecTimer")
		Event_FireGenericEvent("PlayerResurrected")
	end
	
	self.wndResurrect:Show(false)
end

-------- Hide Confirmation window and resurrect where you died -------------
function DeathImproved:OnConfirmResurrectHere()
	if GameLib.GetPlayerUnit() ~= nil then
		GameLib.GetPlayerUnit():Resurrect(RezType.Here, 0)
	end
	
	self.wndResurrectHereConfirm:Show(false)
	self.bDead = false
	
	Apollo.StopTimer("TenthSecTimer")
	Event_FireGenericEvent("PlayerResurrected")
end

-------- Return to Main window ---------------------------------------------
function DeathImproved:OnCancelResurrectHere()
	self.wndResurrectHereConfirm:Show(false)	
	self.wndResurrect:Show(true)
end
------------------------------------//------------------------------
function DeathImproved:OnResurrectCaster()
	if GameLib.GetPlayerUnit() ~= nil then
		GameLib.GetPlayerUnit():Resurrect(RezType.SpellCasterLocation, 0) -- WIP, this should send in the UnitId of the caster
	end
	
	self.wndResurrect:Show(false)
	self.bDead = false
	
	Apollo.StopTimer("TenthSecTimer")
	Event_FireGenericEvent("PlayerResurrected")
end
------------------------------------//------------------------------
function DeathImproved:OnResurrectEldan()
	if GameLib.GetPlayerUnit() ~= nil then
		GameLib.GetPlayerUnit():Resurrect(RezType.Eldan, 0)
	end

	self.wndResurrect:Show(false)
	self.bDead = false
	
	Apollo.StopTimer("TenthSecTimer")
	Event_FireGenericEvent("PlayerResurrected")
end

------------------------------------//------------------------------
function DeathImproved:OnExitInstance()
	self.wndExitConfirm:Show(true)	
	self.wndResurrect:Show(false)
end

function DeathImproved:OnConfirmExit()
	if GameLib.GetPlayerUnit() ~= nil then
		GameLib.GetPlayerUnit():Resurrect(RezType.ExitInstance, 0)
	end

	self.wndExitConfirm:Show(false)	
	self.wndResurrect:Show(false)
	self.bDead = false
	
	Apollo.StopTimer("TenthSecTimer")
	Event_FireGenericEvent("PlayerResurrected")
end

function DeathImproved:OnCancelExit()
	self.wndExitConfirm:Show(false)	
	self.wndResurrect:Show(true)
end

------------------------------------//------------------------------
function DeathImproved:OnForcedResurrection()
	self.wndExitConfirm:Show(false)	
	self.wndResurrect:Show(false)
	self.bDead = false

	Apollo.StopTimer("TenthSecTimer")
	Event_FireGenericEvent("PlayerResurrected")
end

------------------------------------//------------------------------
function DeathImproved:OnScriptResurrection()
	self.wndExitConfirm:Show(false)	
	self.wndResurrectHereConfirm:Show(false)
	self.wndResurrect:Show(false)
	self.bDead = false

	Apollo.StopTimer("TenthSecTimer")
	Event_FireGenericEvent("PlayerResurrected")
end

------------------------------------//------------------------------

function DeathImproved:HideIncapacitationBar()

	self.wndExitConfirm:Show(false)	
	self.wndResurrect:Show(false)
end

function DeathImproved:HelperCalcTimeSecondsMS(fTimeMS)
	local fTime = math.floor(fTimeMS / 1000)
	local fMillis = fTimeMS % 1000
	return string.format("%d.%d", fTime, math.floor(fMillis / 100))
end

function DeathImproved:HelperCalcTimeMS(fTimeMS)
	local fSeconds = fTimeMS / 1000
	local fMillis = fTimeMS % 1000
	local strOutputSeconds = "00"
	if math.floor(fSeconds % 60) >= 10 then
		strOutputSeconds = tostring(math.floor(fSeconds % 60))
	else
		strOutputSeconds = "0" .. math.floor(fSeconds % 60)
	end
	
	return String_GetWeaselString(Apollo.GetString("CRB_TimeMinsToMS"), math.floor(fSeconds / 60), strOutputSeconds, math.floor(fMillis / 100))
end

function DeathImproved:HelperCalcTime(fSeconds)
	local strOutputSeconds = "00"
	if math.floor(fSeconds % 60) >= 10 then
		strOutputSeconds = tostring(math.floor(fSeconds % 60))
	else
		strOutputSeconds = "0" .. math.floor(fSeconds % 60)
	end
	
	return String_GetWeaselString(Apollo.GetString("CRB_TimeMinsToMS"), math.floor(fSeconds / 60), strOutputSeconds)
end

-----------------------------------------------------------------
------ Modified - Needed to look for Caster --------------------
----------------------------------------------------------------
function DeathImproved:OnUnitCreated(unitNew)
	if unitNew:GetType() == "Player" and self.bIncRes then
		if self.bGroupOnly then
			if unitNew:IsInYourGroup() then
				local idUnit = unitNew:GetId()
				self.arGroupMembers[idUnit] = unitNew
			end
		else
			local idUnit = unitNew:GetId()
			self.arGroupMembers[idUnit] = unitNew
		end
	end
end

function DeathImproved:OnUnitDestroyed(unitDead)
	if unitDead:GetType() == "Player" and self.arGroupMembers[unitDead:GetId()] ~= nil then
		self.arGroupMembers[unitDead:GetId()] = nil
	end
end

function DeathImproved:OnFrame()
	if self.bDead and self.arGroupMembers ~= nil and self.bIncRes then
		for idx, unitNew in pairs(self.arGroupMembers) do
			if (unitNew:GetCastName() == "Resuscitate" or unitNew:GetCastName() == "Awaken" or unitNew:GetCastName() == "Resurrect") and unitNew ~= nil then
				local target = unitNew:GetTarget()
				local myself = GameLib.GetPlayerUnit()
				if target:GetName() ==  myself:GetName()then
					local strTimeBeforeResurrectIncoming = self:HelperCalcTimeSecondsMS(unitNew:GetCastDuration() - unitNew:GetCastElapsed())
					if self.strTimeBeforeResurrectIncoming == nil or strTimeBeforeResurrectIncoming < self.strTimeBeforeResurrectIncoming then
						self.wndResurrect:FindChild("ResurrectIncoming.Timer"):SetText(unitNew:GetName() .. " will resurrect you in " .. strTimeBeforeResurrectIncoming .. Apollo.GetString("CRB__seconds"))
						self.wndResurrect:FindChild("ResurrectIncoming.Timer"):Show(true)
						self.strTimeBeforeResurrectIncoming = strTimeBeforeResurrectIncoming
						self.bResurrectIncoming = true
					end
				end
			elseif self.strTimeBeforeResurrectIncoming ~= nil then		-- Res was aborted for any number of reasons
				self.wndResurrect:FindChild("ResurrectIncoming.Timer"):Show(false)
				self.strTimeBeforeResurrectIncoming = nil
				self.bResurrectIncoming = false
			end
		end
	end
end

----------------------------------------------------------------------------
----------- Modified - Handle options from the options window --------------
----------------------------------------------------------------------------
function DeathImproved:OnWakeHereFilter(wndHandler, wndControl)
	if wndHandler == wndControl then
		self.bConfirmWindowFilter = wndControl:GetData()
	end
end

function DeathImproved:OnIncomingResurrectOption(wndHandler, wndControl)
	if wndHandler == wndControl then
		self.bIncRes = wndControl:GetData()
		self.wndDeathOptions:FindChild("GroupMembersContainer"):Show(self.bIncRes)
	end
end

function DeathImproved:OnConfigure()
	if self.wndDeathOptions and self.wndDeathOptions:IsValid() then
		self.wndDeathOptions:Show(not self.wndDeathOptions:IsVisible())
	end
end

function DeathImproved:OnSlashCmd(cmd, args)
	self.wndDeathOptions:Show(true)
end

function DeathImproved:OnGroupMembersOption(wndHandler, wndControl)
	if wndHandler == wndControl then
		self.bGroupOnly = wndControl:GetData()
		if self.bGroupOnly then
			for idx, unit in pairs(self.arGroupMembers) do
				if not unit:IsInYourGroup() then
					self.arGroupMembers[idx] = nil
				end
			end
		end
	end
end

---------------------------------------------------------------------------------------------------
-- DeathImproved instance
---------------------------------------------------------------------------------------------------
local DeathInst = DeathImproved:new()
DeathInst:Init()
